/*
 * Copyright (c) 2017, Texas Instruments Incorporated
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * *  Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * *  Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * *  Neither the name of Texas Instruments Incorporated nor the names of
 *    its contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
 * THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
 * OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 * OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
 * EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

var MyUtil = (function() {
  "use strict";

    return {

        Queue: function() {
            collection = [];
            this.print = function() {
                console.log(collection);
            };
            this.enqueue = function(element) {
                collection.push(element);
            };
            this.dequeue = function() {
                return collection.shift();
            };
            this.front = function() {
                return collection[0];
            };
            this.size = function() {
                return collection.length;
            };
            this.isEmpty = function() {
                return (collection.length === 0);
            };
        },
        
        bsl_update: function(scriptName, hexName) {
            
            // In your gui file, add the lines
            // <link rel="import" href="../ti-tool-bsl/ti-tool-bsl.html">
            //  AND
            // <ti-tool-bsl id="ti_tool_bsl" class="_TI_"></ti-tool-bsl>
            
            // Assume your app folder has a folder called 'fw',
            // and under 'fw' folder, you have script.txt, RAM_BSL_USB.txt,
            // blinkLED_f5529.txt, pass32_default.txt, and pass32_wrong.txt
            // These 5 files are modified copies from an example from bslscripter.
            // In particular, the script.txt is simplifed to remove os knowloedge
            // about file delimiter etc. ( '/' and '..', and '\\' )
            // and make all txt files sit in the same directory.
            var prefix = location.href;
            if (prefix.endsWith('index.html')) { prefix += '/../app/fw/'; }
            else { prefix += '../app/fw/'; } 
            
            // Build file urls
            let scriptfile_url = prefix + scriptName;
            let hexfile_url = prefix + hexName;
            
            // Call bsl to load program and handle feedback and failure
            let BSL = templateObj.$.ti_tool_bsl;
            if (BSL) {
                BSL.customLoad(scriptfile_url, [hexfile_url]).then(x => {
                    // handle feedback from bsl tool, e.g. console.log(x)
                    console.log(x);
                }).catch(x => {
                    // handle failure, e.g. console.log(x)
                    console.log(x);
                });
            } else {
                
            }
            
            //example(scriptfile_url);
        },
        
        
        // REFERENCE: https://stackoverflow.com/questions/10073699/pad-a-number-with-leading-zeros-in-javascript
        pad: function(n, width, z) {
          z = z || '0';
          n = n + '';
          return n.length >= width ? n : new Array(width - n.length + 1).join(z) + n;
        },
        
        // Helper function to fire custom events
        fireEvent: function(eventName, module) {
            
            // Check if module name was provided, if so use the GC log
            if (module) {
                gc.console.log(module, "Firing '" + eventName + "' event");
            } else {
                console.log("Firing '" + eventName + "' event");
            }
            var event = new Event(eventName);
            document.dispatchEvent(event);
        },
        
        // Read data from JSON file
        readJSONFile: function(filePath) {
            return gc.fileCache.readJsonFile(filePath);
        },
        
        // Read data from CSV file
        readCSVFile: function(filePath) {
            return new Promise(function(resolve, reject) {
                gc.fileCache.readTextFile(filePath).then(data => {
                    data = data.replace(/[\n\r]/g, '\n');
                    // TODO: Convert to array
                    resolve(csvFile);
                });
            });
        },
        
        isEmpty: function(obj) {
            for (var key in obj) {
                if (obj.hasOwnProperty(key))
                    return false;
            }
            return true;
        },
        
        setStatusBarProgress: function(percent) {
            let statusBar = document.getElementById("ti_widget_statusbar");
            statusBar.progress = percent;
            statusBar.progressBarEnabled = true;    //NOTE:  Statusbar automatically hides if percent is 0 or 100.
        },
        
        
        findFieldObjectById: function(fieldIdString, registerIdString, regblockIdString) {
            // // REFERENCE: https://confluence.itg.ti.com/pages/viewpage.action?pageId=202419411
            
            // // EXAMPLE: How to binding a dropdown menu to a register field
                
            //     // we then need to ask it for the JSON object that corresponds to the values in the register.json file:
            //     var registerModel = document.querySelector('ti-model-register');
            //     registerModel.getRegisterViewInformation().then(function(deviceObj) {
                    
            //         // filter the register blocks based on id
            //         const regBlockArray = deviceObj.regblocks.filter((regblock) => (regblock.id === "device_configuration"));
            //         if (regBlockArray.length > 0) {
            //             const regBlock = regBlockArray[0];
            //             // filter the registers from that register block based on id
            //             const regArray = regBlock.registers.filter((reg) => (reg.id === "gain1_register"));
            //             if (regArray.length > 0) {
            //                 const reg = regArray[0];
            //                 // filter the bitfields from that register based on id
            //                 const bitfieldArray = reg.fields.filter((bitfield) => (bitfield.id === "pgagain0"));
                            
            //                 if (bitfieldArray.length > 0) {
            //                     const bitfield = bitfieldArray[0];
                                
            //                 }
            //             }
            //         }
            //     });
        },
        
        
        // Reference: https://italonascimento.github.io/applying-a-timeout-to-your-promises/
        promiseTimeout: function(ms, promise) {
            
            // Create a promise that rejects in <ms> milliseconds
            let id;
            let timeout = new Promise((resolve, reject) => {
                id = setTimeout(() => {
                    reject(`Promise timed out in ${ms} ms.`);
                }, +ms);
            });
            
            // Returns a race between our timeout and the passed in promise
            return Promise.race([promise, timeout]).then(result => {
                clearTimeout(id);
                return result;
            });
        },
        
        // Loads and executes a script file
        executeScript: function(scriptName, model) {
            
            // Set default model if none is provided
            model = model || gc.databind.registry.getModel('my_streaming_model'); // Alt: templateObj.$.my_streaming_model;
            
            if (gc.desktop.isDesktop()) {
                
            //let 
                
            }
            
            
            // Read in script and bootloader files
            Q.all([
                gc.fileCache.readTextFile('app/scripts/' + scriptName),
                gc.fileCache.readTextFile('app/scripts/macros.js')
            ]).then(([script, logfile, bootloader]) => {
                
                let bufferLength = null;
                const scripting = model.newScriptInstance(bufferLength);
                
                scripting.load(script, bootloader);
                scripting.main();
            }).fail(e => {
                console.error(e);
            });
        },
        
        // TODO: Create a helper function for binding drop-down list widgets to register fields
        bindDropDownListToFieldValue: function(dropDownlistObj, bitfieldObj) {
            
            // The widget's binding to the register bitfield controls what the selected value and label is
            const widgetOptions = bitfieldObj.widget.options;   // TODO: In the future, check for options on the bitfield object directly (not in the widget)
            
            // Use the options for the bitfield to create the labels and values for the droplist
            const labels = widgetOptions.map((option) => (option.display));
            dropDownlistObj.labels = labels.join("|");
            
            const values = widgetOptions.map((option) => (option.value));
            dropDownlistObj.values = values.join(",");
        },
        
        
        saveTextFileAs: function(textData, fileFilter) {
            if (gc.desktop.isDesktop()) {
                
                fileFilter = fileFilter || "";
                let fileName = "newTextFile" + fileFilter;
                
                gc.File.browseAndSave(textData, fileName, fileFilter, function (result, errorInfo) {
                    if (errorInfo) {
                        templateObj.logMsgAndNotify("error", calledBy + " " + errorInfo, false, true);
                    } else if ((result) && (result.localPath)) {
                        templateObj.$.ti_widget_eventlog_view.log("info", "Text file saved to " + result.localPath);
                    }
                });
                
            } else {
                let mime = 'text/plain;charset=utf-8';
                var blob = new Blob([text], {
                    type: mime
                });
                saveAs(blob, fname);
                templateObj.$.ti_widget_eventlog_view.log("info", calledBy+": "+fileName+" file downloaded");
            }
        }
    };
        
})();


// The register model reads in the .json file when it starts up, so in order to get
            // access to this information, we need to wait for the register model to send a registerDefsChanged
            // event that says it has updated its register model:
            //var registerModel = document.querySelector('ti-model-register');
            //registerModel.addEventListener('registerDefsChanged', function(event) {
                
                // EXAMPLE: How to binding a dropdown menu to a register field
                // REFERENCE: https://confluence.itg.ti.com/pages/viewpage.action?pageId=202419411
                // we then need to ask it for the JSON object that corresponds to the values in the register.json file:
                // registerModel.getRegisterViewInformation().then(function(deviceObj) {
                    
                //     // filter the register blocks based on id
                //     const regBlockArray = deviceObj.regblocks.filter((regblock) => (regblock.id === "device_configuration"));
                //     if (regBlockArray.length > 0) {
                //         const regBlock = regBlockArray[0];
                //         // filter the registers from that register block based on id
                //         const regArray = regBlock.registers.filter((reg) => (reg.id === "gain1_register"));
                //         if (regArray.length > 0) {
                //             const reg = regArray[0];
                //             // filter the bitfields from that register based on id
                //             const bitfieldArray = reg.fields.filter((bitfield) => (bitfield.id === "pgagain0"));
                            
                //             if (bitfieldArray.length > 0) {
                //                 const bitfield = bitfieldArray[0];
                                
                //                 // use the options for the bitfield to create the labels and values for the droplist
                //                 const widgetOptions = bitfield.widget.options;
                                
                //                 const labels = widgetOptions.map((option) => (option.display));
                //                 templateObj.$.ti_widget_droplist_demo.labels = labels.join("|");
                                
                //                 const values = widgetOptions.map((option) => (option.value));
                //                 templateObj.$.ti_widget_droplist_demo.values = values.join(",");
                //                 // the widget's binding to the register bitfield controls what the selected value and label is
                //             }
                //         }
                //     }
                // });
            //});